% loadbv() - load Brain Vision Recorder vhdr-file and return and
%            EEGLAB structure.
%
% Usage:
%  >> EEG = pop_loadbv; % pop-up window mode
%  >> EEG = pop_loadbv(path,hdrfile,srange);
%
% Inputs:
%   path    - Path to files
%   hdrfile - Name of Brain Vision vhdr-file (including extension)
%   srange  - Interval in samples (e.g., [7 42]; default: [1 inf])
%
% Outputs:
%   EEG     - EEGLAB EEG structure
%
% Author: Andreas Widmann & Arnaud Delorme, 2004-

% Revision 1.2  2005/05/11  18:42:00 Andreas Widmann
% Parsing of channel info (fileformat change in Vision Recorder 1.03)
%
% Revision 1.1  2004/12/21  15:19:00 Andreas Widmann
% Added range parameter
%
% Formating for EEGLAB - 11/16/2004 - Arnaud Delorme

function [EEG, com] = loadbv(path,hdrfile,srange)

try, EEG = eeg_emptyset;
catch, end;
com = '';

if nargin < 1
    [hdrfile path] = uigetfile('*', 'Select Brain Vision vhdr-file - pop_loadbv()');
    if hdrfile(1) == 0, EEG = []; return; end;

    drawnow;
    uigeom       = {[1 0.5]};
    uilist       = { { 'style' 'text' 'string' 'Interval in samples (e.g., [7 42]; default: [1 inf]):' }
                     { 'style' 'edit' 'string' '' } };
    result = inputgui( uigeom, uilist, 'pophelp(''pop_loadbv'')', 'Load a Brain Vision Recorder dataset');
    if length( result ) == 0, return; end
    if ~isempty(result{1}),
        srange = eval([ '[' result{1} ']' ]);
    else
        srange = [1 inf];
    end
end;

if isempty(srange),
    srange = [1 inf];
end

if size(srange,2) < 2,
    srange(2) = inf;
end

disp('loadbv(): read and parse header file');
if path(end) ~= filesep
    path = [path filesep];
end
[IN,message] = fopen([path hdrfile]);
if IN == -1
    error(message);
end
hdrraw={};
while ~feof(IN)
    hdrraw = [hdrraw;{fgetl(IN)}];
end
fclose(IN);

if ~isempty(strmatch('[Coordinates]', hdrraw))
    coords = hdrraw(strmatch('[Coordinates]', hdrraw):end);
    hdrraw = hdrraw(1:strmatch('[Coordinates]', hdrraw));
end;

filename = char(strread(hdrraw{strmatch('DataFile',hdrraw)},'DataFile=%s'));
EEG.comments        = [ 'Original file: ' filename ];
mrkfile = char(strread(hdrraw{strmatch('MarkerFile',hdrraw)},'MarkerFile=%s'));
EEG.nbchan = strread(hdrraw{strmatch('NumberOfChannels',hdrraw)},'NumberOfChannels=%d');
EEG.srate = 1000000 / strread(hdrraw{strmatch('SamplingInterval',hdrraw)},'SamplingInterval=%f');
for chan = 1:EEG.nbchan
    temp = strread(hdrraw{strmatch(['Ch' num2str(chan) '='],hdrraw)},'%s','delimiter',',');
    EEG.chanlocs(chan).labels = char(strread(temp{1},['Ch' num2str(chan) '=%s']));
    if length(temp) > 1
        chanlocs(chan).ref = temp{2};
    end;
    if length(temp) > 2
        chanlocs(chan).scale = str2num(temp{3});
    end;
end

% read different binary file formats
% ----------------------------------
disp('loadbv(): read and scale EEG data');
binformat   = char(strread(hdrraw{strmatch('BinaryFormat',hdrraw)},'BinaryFormat=%s'));
if isempty(binformat),
    error('Can only read binary file formats');
end;
switch lower(binformat)
 case 'int_16'       , binformat = 'int16';
 case 'int_32'       , binformat = 'int32';
 case 'ieee_float_32', binformat = 'float';
 otherwise error('Unknown binary format');
end;
[IN,message]=fopen([path filename]);
if IN == -1
    error(message);
end
EEG.data = fread(IN, Inf, binformat);

% reorient data
% -------------
orientation = char(strread(hdrraw{strmatch('DataOrientation',hdrraw)},'DataOrientation=%s'));
if strcmpi(orientation, 'multiplexed')
    EEG.data = reshape(EEG.data, EEG.nbchan, length(EEG.data)/EEG.nbchan);
else
    EEG.data = reshape(EEG.data, length(EEG.data)/EEG.nbchan, EEG.nbchan);
    EEG.data = EEG.data';
end;
EEG.pnts = size(EEG.data,2);
if isfield(EEG.chanlocs, 'scale');
    for chan = 1:EEG.nbchan,
        EEG.data(chan,:) = EEG.data(chan,:) .* chanlocs(chan).scale;
    end
end;

% import marker file
% ------------------
disp('loadbv(): read and parse marker file');
[IN,message] = fopen([path mrkfile]);
if IN == -1
    error(message);
end
mrkraw={};
while ~feof(IN)
    mrkraw = [mrkraw;{fgetl(IN)}];
end
fclose(IN);
EEG.trials = 1;
EEG.xmin   = 0;
EEG.xmax   = (EEG.pnts - 1) / EEG.srate;

if isstr(mrkraw{1}) % non-empty file
    markers = strmatch('Mk' ,mrkraw)';
    for c = 1:size(markers,2),
        [mrkn,mrktype,mrkdesc,EEG.event(c).latency,mrksizepnts,mrkchan,mrktime] = ...
            strread(mrkraw{markers(c)}, 'Mk%d=%s%s%f%d%d%d','delimiter',',');
        EEG.event(c).code        = '';
        if strcmp(mrktype,'New Segment') | strcmp(mrktype,'DC Correction'),
            EEG.event(c).type = 'boundary';
        elseif ~isempty(char(mrkdesc))
            EEG.event(c).type    = char(mrkdesc);
            EEG.event(c).code    = char(mrktype);
        else
            EEG.event(c).type = char(mrktype);
        end
        EEG.event(c).latency = EEG.event(c).latency - srange(1) + 1;
    end
    
    EEG.event = EEG.event(find([EEG.event.latency] >= srange(1) & [EEG.event.latency] <= srange(2)));
    EEG.urevent = EEG.event;
    
    % find if boundaries at homogenous intervals
    % ------------------------------------------
    boundind = strmatch('boundary', { EEG.event.type });
    if ~isempty(boundind)
        boundlat = [ EEG.event(boundind).latency ];
        epochlen = boundlat(1:end-1)-boundlat(2:end);
        if length(unique(epochlen)) == 1
            EEG.trials = length(boundlat);
            EEG.pnts   = -epochlen(1);
            EEG.event(boundind) = [];
            
            % adding epoch field
            % ------------------
            for index = 1:length(EEG.event)
                EEG.event(index).epoch = ceil(EEG.event(index).latency / EEG.pnts);
            end;
            
            % finding minimum time
            % --------------------
            time0ind = strmatch('time 0', lower({ EEG.event.code }));
            if ~isempty(time0ind)
                for index = time0ind(:)'
                    EEG.event(index).type = 'TLE';
                end;
                EEG.xmin = -(EEG.event(time0ind(1)).latency-1)/EEG.srate;
            end;
        end;
    end;
end;

EEG.ref = 'common';

try, EEG = EEG.checkset(EEG);
catch, end

if nargout == 2
    com = sprintf('EEG = pop_loadbv(''%s'', ''%s'', [%d %d]);', path, hdrfile, srange);
end;